@@ -73,7 +73,7 @@
  */
 
 
-#define FUSE_USE_VERSION 31
+#define FUSE_USE_VERSION 34
 
 #include <fuse_lowlevel.h>
 #include <stdio.h>
@@ -273,6 +273,7 @@ int main(int argc, char *argv[]) {
     struct fuse_args args = FUSE_ARGS_INIT(argc, argv);
     struct fuse_session *se;
     struct fuse_cmdline_opts opts;
+    struct fuse_loop_config config;
     pthread_t updater;
     int ret = -1;
 
@@ -321,8 +322,11 @@ int main(int argc, char *argv[]) {
     /* Block until ctrl+c or fusermount -u */
     if (opts.singlethread)
         ret = fuse_session_loop(se);
-    else
-        ret = fuse_session_loop_mt(se, opts.clone_fd);
+    else {
+        config.clone_fd = opts.clone_fd;
+        config.max_idle_threads = opts.max_idle_threads;
+        ret = fuse_session_loop_mt(se, &config);
+    }
 
     fuse_session_unmount(se);
 err_out3:
